@@ -1,5 +1,6 @@
 from django.conf import settings
 from django.utils.timezone import now
+from django.db import DatabaseError, ProgrammingError
 
 from garpix_notify.models.config import NotifyConfig
 from garpix_notify.models.choices import STATE
@@ -18,7 +19,7 @@ class TelegramClient:
             self.TELEGRAM_DISABLE_PAGE_PREVIEW = self.config.telegram_disable_web_page_preview
             self.TELEGRAM_SENDING_WITHOUT_REPLY = self.config.telegram_allow_sending_without_reply
             self.TELEGRAM_TIMEOUT = self.config.telegram_timeout
-        except Exception:
+        except (DatabaseError, ProgrammingError):
             self.IS_TELEGRAM_ENABLED = True
             self.TELEGRAM_PARSE_MODE = getattr(settings, 'TELEGRAM_PARSE_MODE', None)
             self.TELEGRAM_DISABLE_NOTIFICATION = getattr(settings, 'TELEGRAM_DISABLE_NOTIFICATION', False)
@@ -34,13 +35,15 @@ class TelegramClient:
 
     def __send_telegram_client(self):
         import telegram
-        bot = telegram.Bot(token=self.TELEGRAM_API_KEY)
-        parse_mode = self.TELEGRAM_PARSE_MODE if self.TELEGRAM_PARSE_MODE != '' else None
+
         if not self.IS_TELEGRAM_ENABLED:
             self.notify.state = STATE.DISABLED
             self.notify.to_log('Not sent (sending is prohibited by settings)')
             return
 
+        bot = telegram.Bot(token=self.TELEGRAM_API_KEY)
+        parse_mode = self.TELEGRAM_PARSE_MODE if self.TELEGRAM_PARSE_MODE != '' else None
+
         try:
             result = False
             for chunk in self.__chunks(str(self.notify.text), 4096):
